using System;
using System.Collections;
using System.Data;
using System.Data.OleDb;
using System.Text;
using System.Xml;
using Gregor.Cgi;

namespace Gregor.Cgi.Media {

internal class CSheetInfo : CSqlRenderableBase {

    private string m_SqlInsertText;
    private string m_SqlUpdateText;
    private ArrayList m_Inputs;
    private string m_NextPage;

    public CSheetInfo(XmlElement xe) : base(xe){
        XmlAttribute xa;
        xa = xe.Attributes["sql-insert-text"];
        if(xa != null){
            m_SqlInsertText = xa.Value;
        }
        xa = xe.Attributes["sql-update-text"];
        if(xa != null){
            m_SqlUpdateText = xa.Value;
        }
        m_Inputs = new ArrayList();
        foreach(XmlNode xnChild in xe.ChildNodes){
            if(xnChild is XmlElement){
                XmlElement xeChild = (XmlElement) xnChild;
                switch(xeChild.Name){
                    case "input":
                        CInputInfo input = new CInputInfo(xeChild);
                        m_Inputs.Add(input);
                        break;
                }
            }
        }
        xa = xe.Attributes["next-page"];
        if(xa != null){
            m_NextPage = xa.Value;
        }
    }

    public string SqlInsertText{
        get{
            return m_SqlInsertText;
        }
    }

    public string SqlUpdateText{
        get{
            return m_SqlUpdateText;
        }
    }

    public ArrayList Inputs{
        get{
            return m_Inputs;
        }
    }

    public string NextPage{
        get{
            return m_NextPage;
        }
    }

} // class CSheetInfo

internal abstract class CSheetRendererBase : CSqlRendererBase {

    private CSheetInfo m_Sheet;

    protected CSheetRendererBase(CSheetInfo sheet) : base(sheet){
        m_Sheet = sheet;
    }

    public CSheetInfo Sheet{
        get{
            return m_Sheet;
        }
    }

    protected override void DoRender(CRenderingContext context, IDbCommand cmd){
        COut.WriteHtmlHeading(context.Level, "Sheet: " + this.Renderable.Title);
        this.RenderTableOpener(context);
        if(cmd != null){
            IDataReader reader = null;
            try{
                reader = cmd.ExecuteReader();
                if(reader.Read()){
                    this.RenderTableContents(context, reader);
                    if(reader.Read()){
                        throw new Exception("Query must yield exactly one record.");
                    }
                }else{
                    throw new Exception("Cannot read from query.");
                }
            }finally{
                if(reader != null){
                    reader.Close();
                }
            }
        }else{
            this.RenderTableContents(context, null);
        }
        this.RenderTableCloser(context);
    }

    protected virtual void RenderTableOpener(CRenderingContext context){
        COut.WriteHtmlTableOpener();
    }

    protected abstract void RenderTableContents(CRenderingContext context, IDataRecord record);

    protected virtual void RenderTableCloser(CRenderingContext context){
        COut.WriteHtmlTableCloser();
    }

} // class CSheetRendererBase

internal class CDefaultSheetRenderer : CSheetRendererBase {

    public CDefaultSheetRenderer(CSheetInfo sheet) : base(sheet){
        // OK
    }

    protected override void RenderTableContents(CRenderingContext context, IDataRecord record){
        for(int iField = 0; iField < record.FieldCount; iField++){
            COut.WriteHtmlTableRowOpener();
            COut.WriteHtmlTableDataCell(CHelper.FormatValue(record.GetName (iField)), "Left");
            COut.WriteHtmlTableDataCell(CHelper.FormatValue(record.GetValue(iField)));
            COut.WriteHtmlTableRowCloser();
        }
    }

} // class CDefaultSheetRenderer

internal abstract class CInsertUpdateSheetRendererBase : CSheetRendererBase {

    protected CInsertUpdateSheetRendererBase(CSheetInfo sheet) : base(sheet) {
        // OK
    }

    protected override void RenderTableOpener(CRenderingContext context){
        COut.WriteHtmlFormOpener(C.EXE_NAME);
        COut.WriteHtmlHiddenField(this.GetSheetAction(), this.Sheet.Name);
        base.RenderTableOpener(context);
    }

    protected override void RenderTableContents(CRenderingContext context, IDataRecord record){
        foreach(CInputInfo input in this.Sheet.Inputs){
            IRenderer renderer = input.GetRenderer();
            renderer.Render(context);
        }
    }

    protected override void RenderTableCloser(CRenderingContext context){
        COut.WriteHtmlTableRowOpener();
        COut.WriteHtmlTableDataCell("&nbsp;");
        COut.WriteHtmlTableDataCellOpener();
        COut.WriteHtmlSubmitButton();
        COut.WriteHtmlTableDataCellCloser();
        COut.WriteHtmlTableRowCloser();
        base.RenderTableCloser(context);
        COut.WriteHtmlFormCloser();
    }

    protected abstract string GetSheetAction();

} // class CInsertUpdateSheetRendererBase

internal class CInsertSheetRenderer : CInsertUpdateSheetRendererBase {

    public CInsertSheetRenderer(CSheetInfo sheet) : base(sheet) {
        // OK
    }

    protected override IDbCommand CreateCommand(CRenderingContext context){
        return null;
    }

    protected override string GetSheetAction(){
        return "insert";
    }

} // class CInsertSheetRenderer

internal class CUpdateSheetRenderer : CInsertUpdateSheetRendererBase {

    public CUpdateSheetRenderer(CSheetInfo sheet) : base(sheet) {
        // OK
    }

    protected override void RenderTableContents(CRenderingContext context, IDataRecord record){
        foreach(CKeyInfo key in this.Sheet.Keys){
            IRenderer renderer = key.GetRenderer();
            renderer.Render(context);
        }
        foreach(CInputInfo input in this.Sheet.Inputs){
            IRenderer renderer = input.GetRenderer();
            string sValue = record[input.Name].ToString();
            context.SetValue(input.Name, sValue);
            renderer.Render(context);
        }
    }

    protected override string GetSheetAction(){
        return "update";
    }

} // class CUpdateSheetRenderer

} // namespace Gregor.Cgi.Media