using System;
using System.Collections;
using System.Data;
using System.Data.OleDb;
using System.Text;
using System.Xml;
using Gregor.Cgi;

namespace Gregor.Cgi.Media {

internal class CTableInfo : CSqlRenderableBase {

    private ArrayList m_LinkRules;

    public CTableInfo(XmlElement xe): base(xe){
        m_LinkRules = new ArrayList();
        foreach(XmlNode xnChild in xe){
            if(xnChild is XmlElement){
                XmlElement xeChild = (XmlElement) xnChild;
                if(xeChild.Name.Equals("linkrule")){
                    CLinkRule rule = new CLinkRule(xeChild);
                    m_LinkRules.Add(rule);
                }
            }
        }
    }

    public ArrayList LinkRules{
        get{
            return m_LinkRules;
        }
    }

} // class CTableInfo

internal class CLinkRule {

    private string m_HotColumn;
    private string m_FromColumn;
    private string m_ToPage;
    private string m_ToColumn;

    public CLinkRule(XmlElement xe){
        m_HotColumn = xe.Attributes["hot-column"].Value;
        m_FromColumn = xe.Attributes["from-column"].Value;
        m_ToPage = xe.Attributes["to-page"].Value;
        m_ToColumn = xe.Attributes["to-column"].Value;
    }

    public string HotColumn{
        get{
            return m_HotColumn;
        }
    }

    public string FromColumn{
        get{
            return m_FromColumn;
        }
    }

    public string ToPage{
        get{
            return m_ToPage;
        }
    }

    public string ToColumn{
        get{
            return m_ToColumn;
        }
    }

} // class CLinkRule

internal abstract class CTableRendererBase : CSqlRendererBase {

    private CTableInfo m_Table;

    protected CTableRendererBase(CTableInfo table) : base(table){
        m_Table = table;
    }

    public CTableInfo Table{
        get{
            return m_Table;
        }
    }

    protected override void DoRender(CRenderingContext context, IDbCommand cmd){
        COut.WriteHtmlHeading(context.Level, "Table: " + this.Table.Title);
        IDataReader reader = null;
        try{
            reader = cmd.ExecuteReader();
            this.RenderTableOpener();
            this.RenderTableHead(reader);
            while(reader.Read()){
                this.RenderTableRow(reader);
            }
            this.RenderTableCloser();
        }finally{
            if(reader != null){
                reader.Close();
            }
        }
    }

    protected virtual void RenderTableOpener(){
        COut.WriteHtmlTableOpener();
    }

    protected virtual void RenderTableHead(IDataReader reader){
        COut.WriteHtmlTableHeadOpener();
        for(int iField = 0; iField < reader.FieldCount; iField++){
            COut.WriteHtmlTableHeadCell(reader.GetName(iField));
        }
        COut.WriteHtmlTableHeadCloser();
    }

    protected abstract void RenderTableRow(IDataReader reader);

    protected virtual void RenderTableRowOpener(){
        COut.WriteHtmlTableRowOpener();
    }

    protected virtual void RenderTableDataCell(string sText){
        COut.WriteHtmlTableDataCell(sText);
    }

    protected virtual void RenderTableRowCloser(){
        COut.WriteHtmlTableRowCloser();
    }

    protected virtual void RenderTableCloser(){
        COut.WriteHtmlTableCloser();
    }

} // class CTableRendererBase

internal class CLinkingTableRenderer : CTableRendererBase {

    public CLinkingTableRenderer(CTableInfo table) : base(table){
        // OK
    }

    protected override void RenderTableRow(IDataReader reader){
        this.RenderTableRowOpener();
        for(int iField = 0; iField < reader.FieldCount; iField++){
            object oCellValue = null;
            string sColumnName = reader.GetName(iField);
            object oColumnValue = reader.GetValue(iField);
            CLinkRule rule = this.FindLinkRule(sColumnName);
            if(rule != null){
                string sLinkId = reader[rule.FromColumn].ToString();
                oCellValue = CHelper.CreatePageLink(oColumnValue, rule.ToPage, rule.ToColumn, sLinkId);
            }else{
                oCellValue = oColumnValue;
            }
            this.RenderTableDataCell(CHelper.FormatValue(oCellValue));
        }
        this.RenderTableRowCloser();
    }

    protected CLinkRule FindLinkRule(string sColumnName){
        CLinkRule ret = null;
        foreach(CLinkRule rule in this.Table.LinkRules){
            if(rule.HotColumn.Equals(sColumnName)){
                ret = rule;
                break;
            }
        }
        return ret;
    }

} // class CLinkingTableRenderer

internal class CShortLinkingTableRenderer : CLinkingTableRenderer {

    public CShortLinkingTableRenderer(CTableInfo table) : base(table){
        // OK
    }

    protected override void RenderTableOpener(){
        COut.W("<p>");
    }

    protected override void RenderTableHead(IDataReader reader){
        return;
    }

    protected override void RenderTableRowOpener(){
        return;
    }

    protected override void RenderTableDataCell(string sText){
        if(false == CHelper.IsNumeric(sText)){
            COut.W(sText + "&nbsp;&nbsp;");
        }
    }

    protected override void RenderTableRowCloser(){
        return;
    }

    protected override void RenderTableCloser(){
        COut.W("</p>");
    }

} // class CShortLinkingTableRenderer

} // namespace Gregor.Cgi.Media